# -*- coding: utf-8 -*-

import ldap_inp
from ldap_inp import LdapEleve

from .models import PortailUser
from django_cas_ng.backends import CASBackend


class PortailAuthBackend(CASBackend):
    def authenticate(self, request, ticket, service):
        """
        On encapsule la fonction authenticate de CASBackend parce que celle-ci
        nous retourne un objet de type auth.User (nous n'avons jamais modifié
        la variable AUTH_USER_MODEL et maintenant il est difficile de la changer
        en cours de route: https://docs.djangoproject.com/en/3.2/topics/auth/customizing/#substituting-a-custom-user-model).

        Nos views s'attendent à ce que `type(request.user) = PortailUser`.
        Donc on convertit l'object User en l'objet PortailUser équivalent
        (en le créant si nécessaire, lors de la première connexion par exemple).
        (Il y a une équivalence car PortailUser dérive de User, et qu'on ne manipule
        que des Portailuser dans le reste du code. En pratique django a créé une table
        portailuser_portailuser avec une colonne user_ptr qui pointe vers la table
        auth_user. Pour retrouver un utilisateur, on peut donc utiliser au choix
        le nom d'utilisateur - stocké directement dans la table auth_user - ou
        l'identifiant primaire de la table auth_user - stocké dans 'user_ptr'
        dans la table portailuser_portailuser).
        """
        res = super().authenticate(request, ticket, service)
        if res is None:
            return None

        try:
            res = PortailUser.objects.get(username=res.username)
        except PortailUser.DoesNotExist:
            res = self.configure_user(res)

        res.is_superuser = res.ldap_is_superuser()
        res.is_staff = res.ldap_is_staff()

        res.save()

        return res

    def configure_user(self, user):
        """
        Crée l'utilisateur dans la DB SQL s'il n'existe pas, en partant du compte LDAP associé.
        """
        try:
            ldap_user = LdapEleve.objects.get(uid=user.username)
        except ldap_inp.conn.ConnectionFailure:
            print("Serveur LDAP introuvable...")
            raise
        except LdapEleve.DoesNotExist:
            print("The CAS found a user, but the LDAP doesn't know about it, race condition ?")
            return None

        try:
            user = PortailUser.objects.get(username=user.username)
        except PortailUser.DoesNotExist:
            portail_user = PortailUser.objects.create_from_ldap(ldap_user, user)
            portail_user.save()

        return user

    def get_user(self, user_id):
        """
        Return l'object PortailUser associé à la clé primaire 'id' de auth.User.
        """
        try:
            return PortailUser.objects.get(user_ptr=user_id)
        except PortailUser.DoesNotExist:
            return None
